@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -58,13 +58,13 @@ class CVSItem(object):
   __slots__ = [
       'id',
       'cvs_file',
-      'revision_recorder_token',
+      'revision_reader_token',
       ]
 
-  def __init__(self, id, cvs_file, revision_recorder_token):
+  def __init__(self, id, cvs_file, revision_reader_token):
     self.id = id
     self.cvs_file = cvs_file
-    self.revision_recorder_token = revision_recorder_token
+    self.revision_reader_token = revision_reader_token
 
   def __eq__(self, other):
     return self.id == other.id
@@ -199,8 +199,23 @@ class CVSRevision(CVSItem):
         revision.  This member is set in FilterSymbolsPass; before
         then, it is None.
 
-    revision_recorder_token -- (arbitrary) a token that can be set by
-        RevisionRecorder for the later use of RevisionReader.
+    properties -- (dict) the file properties that vary from revision
+        to revision.  The get_properties() method combines the
+        properties found in SELF.cvs_file.properties with those here;
+        the latter take precedence.  Keys are strings.  Values are
+        strings (indicating the property value) or None (indicating
+        that the property should be left unset, even if it is set in
+        SELF.cvs_file.properties).  Different backends can use
+        properties for different purposes; for cvs2svn these become
+        SVN versioned properties.  Properties whose names start with
+        underscore are reserved for internal cvs2svn purposes.
+
+    properties_changed -- (None or bool) Will this CVSRevision's
+        get_properties() method return a different value than the same
+        call of the predecessor revision?
+
+    revision_reader_token -- (arbitrary) a token that can be set by
+        RevisionCollector for the later use of RevisionReader.
 
   """
 
@@ -221,20 +236,24 @@ class CVSRevision(CVSItem):
       'branch_commit_ids',
       'opened_symbols',
       'closed_symbols',
+      'properties',
+      'properties_changed',
       ]
 
-  def __init__(self,
-               id, cvs_file,
-               timestamp, metadata_id,
-               prev_id, next_id,
-               rev, deltatext_exists,
-               lod, first_on_branch_id, ntdbr,
-               ntdbr_prev_id, ntdbr_next_id,
-               tag_ids, branch_ids, branch_commit_ids,
-               revision_recorder_token):
+  def __init__(
+        self,
+        id, cvs_file,
+        timestamp, metadata_id,
+        prev_id, next_id,
+        rev, deltatext_exists,
+        lod, first_on_branch_id, ntdbr,
+        ntdbr_prev_id, ntdbr_next_id,
+        tag_ids, branch_ids, branch_commit_ids,
+        revision_reader_token,
+        ):
     """Initialize a new CVSRevision object."""
 
-    CVSItem.__init__(self, id, cvs_file, revision_recorder_token)
+    CVSItem.__init__(self, id, cvs_file, revision_reader_token)
 
     self.timestamp = timestamp
     self.metadata_id = metadata_id
@@ -252,6 +271,8 @@ class CVSRevision(CVSItem):
     self.branch_commit_ids = branch_commit_ids
     self.opened_symbols = None
     self.closed_symbols = None
+    self.properties = None
+    self.properties_changed = None
 
   def _get_cvs_path(self):
     return self.cvs_file.cvs_path
@@ -279,7 +300,8 @@ class CVSRevision(CVSItem):
         self.ntdbr_prev_id, self.ntdbr_next_id,
         self.tag_ids, self.branch_ids, self.branch_commit_ids,
         self.opened_symbols, self.closed_symbols,
-        self.revision_recorder_token,
+        self.properties, self.properties_changed,
+        self.revision_reader_token,
         )
 
   def __setstate__(self, data):
@@ -294,10 +316,43 @@ class CVSRevision(CVSItem):
      self.ntdbr_prev_id, self.ntdbr_next_id,
      self.tag_ids, self.branch_ids, self.branch_commit_ids,
      self.opened_symbols, self.closed_symbols,
-     self.revision_recorder_token) = data
-    self.cvs_file = Ctx()._cvs_file_db.get_file(cvs_file_id)
+     self.properties, self.properties_changed,
+     self.revision_reader_token) = data
+    self.cvs_file = Ctx()._cvs_path_db.get_path(cvs_file_id)
     self.lod = Ctx()._symbol_db.get_symbol(lod_id)
 
+  def get_properties(self):
+    """Return all of the properties needed for this CVSRevision.
+
+    Combine SELF.cvs_file.properties and SELF.properties to get the
+    final properties needed for this CVSRevision.  (The properties in
+    SELF have precedence.)  Return the properties as a map {key :
+    value}, where keys and values are both strings.  (Entries with
+    value == None are omitted.)  Different backends can use properties
+    for different purposes; for cvs2svn these become SVN versioned
+    properties.  Properties whose names start with underscore are
+    reserved for internal cvs2svn purposes."""
+
+    properties = self.cvs_file.properties.copy()
+    properties.update(self.properties)
+
+    for (k,v) in properties.items():
+      if v is None:
+        del properties[k]
+
+    return properties
+
+  def get_property(self, name, default=None):
+    """Return a particular property for this CVSRevision.
+
+    This is logically the same as SELF.get_properties().get(name,
+    default) but implemented more efficiently."""
+
+    if name in self.properties:
+      return self.properties[name]
+    else:
+      return self.cvs_file.properties.get(name, default)
+
   def get_effective_prev_id(self):
     """Return the ID of the effective predecessor of this item.
 
@@ -599,8 +654,8 @@ class CVSSymbol(CVSItem):
         CVSRevision, but can be changed to a CVSBranch via parent
         adjustment in FilterSymbolsPass.
 
-    revision_recorder_token -- (arbitrary) a token that can be set by
-        RevisionRecorder for the later use of RevisionReader.
+    revision_reader_token -- (arbitrary) a token that can be set by
+        RevisionCollector for the later use of RevisionReader.
 
   """
 
@@ -612,11 +667,11 @@ class CVSSymbol(CVSItem):
 
   def __init__(
       self, id, cvs_file, symbol, source_lod, source_id,
-      revision_recorder_token
+      revision_reader_token,
       ):
     """Initialize a CVSSymbol object."""
 
-    CVSItem.__init__(self, id, cvs_file, revision_recorder_token)
+    CVSItem.__init__(self, id, cvs_file, revision_reader_token)
 
     self.symbol = symbol
     self.source_lod = source_lod
@@ -677,8 +732,8 @@ class CVSBranch(CVSSymbol):
         branch.  This member is set in FilterSymbolsPass; before then,
         it is None.
 
-    revision_recorder_token -- (arbitrary) a token that can be set by
-        RevisionRecorder for the later use of RevisionReader.
+    revision_reader_token -- (arbitrary) a token that can be set by
+        RevisionCollector for the later use of RevisionReader.
 
   """
 
@@ -693,13 +748,13 @@ class CVSBranch(CVSSymbol):
   def __init__(
       self, id, cvs_file, symbol, branch_number,
       source_lod, source_id, next_id,
-      revision_recorder_token,
+      revision_reader_token,
       ):
     """Initialize a CVSBranch."""
 
     CVSSymbol.__init__(
         self, id, cvs_file, symbol, source_lod, source_id,
-        revision_recorder_token
+        revision_reader_token,
         )
     self.branch_number = branch_number
     self.next_id = next_id
@@ -714,7 +769,7 @@ class CVSBranch(CVSSymbol):
         self.source_lod.id, self.source_id, self.next_id,
         self.tag_ids, self.branch_ids,
         self.opened_symbols,
-        self.revision_recorder_token,
+        self.revision_reader_token,
         )
 
   def __setstate__(self, data):
@@ -724,9 +779,9 @@ class CVSBranch(CVSSymbol):
         source_lod_id, self.source_id, self.next_id,
         self.tag_ids, self.branch_ids,
         self.opened_symbols,
-        self.revision_recorder_token,
+        self.revision_reader_token,
         ) = data
-    self.cvs_file = Ctx()._cvs_file_db.get_file(cvs_file_id)
+    self.cvs_file = Ctx()._cvs_path_db.get_path(cvs_file_id)
     self.symbol = Ctx()._symbol_db.get_symbol(symbol_id)
     self.source_lod = Ctx()._symbol_db.get_symbol(source_lod_id)
 
@@ -819,8 +874,8 @@ class CVSTag(CVSSymbol):
         be changed to a CVSBranch via parent adjustment in
         FilterSymbolsPass.
 
-    revision_recorder_token -- (arbitrary) a token that can be set by
-        RevisionRecorder for the later use of RevisionReader.
+    revision_reader_token -- (arbitrary) a token that can be set by
+        RevisionCollector for the later use of RevisionReader.
 
   """
 
@@ -828,28 +883,28 @@ class CVSTag(CVSSymbol):
 
   def __init__(
       self, id, cvs_file, symbol, source_lod, source_id,
-      revision_recorder_token,
+      revision_reader_token,
       ):
     """Initialize a CVSTag."""
 
     CVSSymbol.__init__(
         self, id, cvs_file, symbol, source_lod, source_id,
-        revision_recorder_token,
+        revision_reader_token,
         )
 
   def __getstate__(self):
     return (
         self.id, self.cvs_file.id, self.symbol.id,
         self.source_lod.id, self.source_id,
-        self.revision_recorder_token,
+        self.revision_reader_token,
         )
 
   def __setstate__(self, data):
     (
         self.id, cvs_file_id, symbol_id, source_lod_id, self.source_id,
-        self.revision_recorder_token,
+        self.revision_reader_token,
         ) = data
-    self.cvs_file = Ctx()._cvs_file_db.get_file(cvs_file_id)
+    self.cvs_file = Ctx()._cvs_path_db.get_path(cvs_file_id)
     self.symbol = Ctx()._symbol_db.get_symbol(symbol_id)
     self.source_lod = Ctx()._symbol_db.get_symbol(source_lod_id)
 
