@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -31,7 +31,7 @@ from cvs2svn_lib.cvs_item import CVSTagNoop
 from cvs2svn_lib.cvs_file_items import CVSFileItems
 from cvs2svn_lib.serializer import Serializer
 from cvs2svn_lib.serializer import PrimedPickleSerializer
-from cvs2svn_lib.database import IndexedStore
+from cvs2svn_lib.indexed_database import IndexedStore
 
 
 cvs_item_primer = (
