@@ -17,11 +17,6 @@
 """Represent CVSRevision metadata."""
 
 
-from __future__ import generators
-
-from cvs2svn_lib.boolean import *
-
-
 class Metadata(object):
   def __init__(self, id, author, log_msg):
     self.id = id
