@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -25,7 +25,7 @@ from cvs2svn_lib.artifact_manager import artifact_manager
 from cvs2svn_lib.record_table import SignedIntegerPacker
 from cvs2svn_lib.record_table import RecordTable
 from cvs2svn_lib.serializer import PrimedPickleSerializer
-from cvs2svn_lib.database import IndexedDatabase
+from cvs2svn_lib.indexed_database import IndexedDatabase
 from cvs2svn_lib.svn_commit import SVNRevisionCommit
 from cvs2svn_lib.svn_commit import SVNInitialProjectCommit
 from cvs2svn_lib.svn_commit import SVNPrimaryCommit
@@ -52,7 +52,7 @@ class PersistenceManager:
   def __init__(self, mode):
     self.mode = mode
     if mode not in (DB_OPEN_NEW, DB_OPEN_READ):
-      raise RuntimeError, "Invalid 'mode' argument to PersistenceManager"
+      raise RuntimeError("Invalid 'mode' argument to PersistenceManager")
     primer = (
         SVNInitialProjectCommit,
         SVNPrimaryCommit,
@@ -88,8 +88,9 @@ class PersistenceManager:
     CVS_REVS and record associated attributes."""
 
     if self.mode == DB_OPEN_READ:
-      raise RuntimeError, \
+      raise RuntimeError(
           'Write operation attempted on read-only PersistenceManager'
+          )
 
     self.svn_commit_db[svn_commit.revnum] = svn_commit
 
