@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -17,7 +17,6 @@
 """This module contains database facilities used by cvs2svn."""
 
 
-import re
 import os
 import cPickle
 
@@ -54,6 +53,7 @@ class Project(object):
         self, id, project_cvs_repos_path,
         initial_directories=[],
         symbol_transforms=None,
+        exclude_paths=[],
         ):
     """Create a new Project record.
 
@@ -66,7 +66,13 @@ class Project(object):
 
     SYMBOL_TRANSFORMS is an iterable of SymbolTransform instances
     which will be used to transform any symbol names within this
-    project."""
+    project.
+
+    EXCLUDE_PATHS is an iterable of paths that should be excluded from
+    the conversion.  The paths should be relative to
+    PROJECT_CVS_REPOS_PATH and use slashes ('/').  Paths for
+    individual files should include the ',v' extension.
+    """
 
     self.id = id
 
@@ -79,11 +85,6 @@ class Project(object):
         self.determine_repository_root(
             os.path.abspath(self.project_cvs_repos_path))
 
-    # A regexp matching project_cvs_repos_path plus an optional separator:
-    self.project_prefix_re = re.compile(
-        r'^' + re.escape(self.project_cvs_repos_path)
-        + r'(' + re.escape(os.sep) + r'|$)')
-
     # The SVN directories to add when the project is first created:
     self._initial_directories = []
 
@@ -106,6 +107,8 @@ class Project(object):
 
     self.symbol_transform = CompoundSymbolTransform(symbol_transforms)
 
+    self.exclude_paths = set(exclude_paths)
+
     # The ID of the Trunk instance for this Project.  This member is
     # filled in during CollectRevsPass.
     self.trunk_id = None
@@ -183,7 +186,7 @@ class Project(object):
     This method can only be called after self.root_cvs_directory_id
     has been initialized in CollectRevsPass."""
 
-    return Ctx()._cvs_file_db.get_file(self.root_cvs_directory_id)
+    return Ctx()._cvs_path_db.get_path(self.root_cvs_directory_id)
 
   def get_initial_directories(self):
     """Generate the project's initial SVN directories.
@@ -194,7 +197,7 @@ class Project(object):
     # Yield the path of the Trunk symbol for this project (which might
     # differ from the one passed to the --trunk option because of
     # SymbolStrategyRules).  The trunk path might be '' during a
-    # trunk-only conversion, but that is OK because DumpfileDelegate
+    # trunk-only conversion, but that is OK because DumpstreamDelegate
     # considers that directory to exist already and will therefore
     # ignore it:
     yield self.get_trunk().base_path
@@ -208,12 +211,16 @@ class Project(object):
 
 def read_projects(filename):
   retval = {}
-  for project in cPickle.load(open(filename, 'rb')):
+  f = open(filename, 'rb')
+  for project in cPickle.load(f):
     retval[project.id] = project
+  f.close()
   return retval
 
 
 def write_projects(filename):
-  cPickle.dump(Ctx()._projects.values(), open(filename, 'wb'), -1)
+  f = open(filename, 'wb')
+  cPickle.dump(Ctx()._projects.values(), f, -1)
+  f.close()
 
 
