@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2009 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -17,7 +17,7 @@
 """This module contains the RepositoryMirror class and supporting classes.
 
 RepositoryMirror represents the skeleton of a versioned file tree with
-multiple lines of development ('LODs').  It records the presence of
+multiple lines of development ('LODs').  It records the presence or
 absence of files and directories, but not their contents.  Given three
 values (revnum, lod, cvs_path), it can tell you whether the specified
 CVSPath existed on the specified LOD in the given revision number.
@@ -137,14 +137,14 @@ import bisect
 from cvs2svn_lib import config
 from cvs2svn_lib.common import DB_OPEN_NEW
 from cvs2svn_lib.common import InternalError
-from cvs2svn_lib.log import Log
+from cvs2svn_lib.log import logger
 from cvs2svn_lib.context import Ctx
-from cvs2svn_lib.cvs_file import CVSFile
-from cvs2svn_lib.cvs_file import CVSDirectory
+from cvs2svn_lib.cvs_path import CVSFile
+from cvs2svn_lib.cvs_path import CVSDirectory
 from cvs2svn_lib.key_generator import KeyGenerator
 from cvs2svn_lib.artifact_manager import artifact_manager
 from cvs2svn_lib.serializer import MarshalSerializer
-from cvs2svn_lib.database import IndexedDatabase
+from cvs2svn_lib.indexed_database import IndexedDatabase
 
 
 class RepositoryMirrorError(Exception):
@@ -531,7 +531,7 @@ class LODHistory(object):
     id = self.ids[index]
 
     if id is None:
-      raise KeyError()
+      raise KeyError(revnum)
 
     return id
 
@@ -563,7 +563,7 @@ class LODHistory(object):
     ceased to exist in REVNUM."""
 
     if revnum < self.revnums[-1]:
-      raise KeyError()
+      raise KeyError(revnum)
     elif revnum == self.revnums[-1]:
       # This is an attempt to overwrite an entry that was already
       # updated during this revision.  Don't allow the replacement
@@ -613,7 +613,7 @@ class _NodeDatabase(object):
   MIN_CACHE_LIMIT = 5000
 
   def __init__(self):
-    self.cvs_file_db = Ctx()._cvs_file_db
+    self.cvs_path_db = Ctx()._cvs_path_db
     self.db = IndexedDatabase(
         artifact_manager.get_temp_file(config.MIRROR_NODES_STORE),
         artifact_manager.get_temp_file(config.MIRROR_NODES_INDEX_TABLE),
@@ -630,7 +630,7 @@ class _NodeDatabase(object):
     # The number of directories in the repository:
     num_dirs = len([
         cvs_path
-        for cvs_path in self.cvs_file_db.itervalues()
+        for cvs_path in self.cvs_path_db.itervalues()
         if isinstance(cvs_path, CVSDirectory)
         ])
 
@@ -642,7 +642,7 @@ class _NodeDatabase(object):
   def _load(self, items):
     retval = {}
     for (id, value) in items:
-      retval[self.cvs_file_db.get_file(id)] = value
+      retval[self.cvs_path_db.get_path(id)] = value
     return retval
 
   def _dump(self, node):
@@ -676,7 +676,7 @@ class _NodeDatabase(object):
       # The size of the cache has exceeded the threshold.  Discard the
       # old cache values (but still store the new nodes into the
       # cache):
-      Log().debug('Clearing node cache')
+      logger.debug('Clearing node cache')
       self._cache.clear()
 
     data = {}
