@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -36,8 +36,6 @@ There are five types of SVNCommits:
 """
 
 
-import textwrap
-
 from cvs2svn_lib.common import InternalError
 from cvs2svn_lib.context import Ctx
 from cvs2svn_lib.symbol import Branch
@@ -47,9 +45,6 @@ from cvs2svn_lib.symbol import Tag
 class SVNCommit:
   """This represents one commit to the Subversion Repository."""
 
-  # textwrap.TextWrapper instance to be used for wrapping log messages:
-  text_wrapper = textwrap.TextWrapper(width=76)
-
   def __init__(self, date, revnum):
     """Instantiate an SVNCommit.
 
@@ -144,7 +139,7 @@ class SVNInitialProjectCommit(SVNCommit):
     return Ctx().username
 
   def get_log_msg(self):
-    return self.text_wrapper.fill(
+    return Ctx().text_wrapper.fill(
         Ctx().initial_project_commit_message % {}
         )
 
@@ -210,7 +205,7 @@ class SVNRevisionCommit(SVNCommit):
     retval = []
     retval.append(SVNCommit.get_warning_summary(self) + '  Related files:')
     for cvs_rev in self.cvs_revs:
-      retval.append('  ' + cvs_rev.cvs_file.filename)
+      retval.append('  ' + cvs_rev.cvs_file.rcs_path)
     return '\n'.join(retval)
 
   def __str__(self):
@@ -281,7 +276,7 @@ class SVNPostCommit(SVNRevisionCommit):
   def get_log_msg(self):
     """Return a manufactured log message for this commit."""
 
-    return self.text_wrapper.fill(
+    return Ctx().text_wrapper.fill(
         Ctx().post_commit_message % {'revnum' : self.motivating_revnum}
         )
 
@@ -330,7 +325,7 @@ class SVNSymbolCommit(SVNCommit):
   def get_log_msg(self):
     """Return a manufactured log message for this commit."""
 
-    return self.text_wrapper.fill(
+    return Ctx().text_wrapper.fill(
         Ctx().symbol_commit_message % {
             'symbol_type' : self._get_symbol_type(),
             'symbol_name' : self.symbol.name,
