@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -21,7 +21,7 @@ import re
 from cvs2svn_lib.common import FatalError
 from cvs2svn_lib.common import path_join
 from cvs2svn_lib.common import normalize_svn_path
-from cvs2svn_lib.log import Log
+from cvs2svn_lib.log import logger
 from cvs2svn_lib.symbol import Trunk
 from cvs2svn_lib.symbol import TypedSymbol
 from cvs2svn_lib.symbol import Branch
@@ -86,7 +86,7 @@ class _RegexpStrategyRule(StrategyRule):
     ACTION(name, id); otherwise it returns SYMBOL unchanged."""
 
     try:
-      self.regexp = re.compile('^' + pattern + '$')
+      self.regexp = re.compile('^(?:' + pattern + ')$')
     except re.error:
       raise FatalError("%r is not a valid regexp." % (pattern,))
 
@@ -112,7 +112,7 @@ class ForceBranchRegexpStrategyRule(_RegexpStrategyRule):
     _RegexpStrategyRule.__init__(self, pattern, Branch)
 
   def log(self, symbol):
-    Log().verbose(
+    logger.verbose(
         'Converting symbol %s as a branch because it matches regexp "%s".'
         % (symbol, self.regexp.pattern,)
         )
@@ -125,7 +125,7 @@ class ForceTagRegexpStrategyRule(_RegexpStrategyRule):
     _RegexpStrategyRule.__init__(self, pattern, Tag)
 
   def log(self, symbol):
-    Log().verbose(
+    logger.verbose(
         'Converting symbol %s as a tag because it matches regexp "%s".'
         % (symbol, self.regexp.pattern,)
         )
@@ -138,7 +138,7 @@ class ExcludeRegexpStrategyRule(_RegexpStrategyRule):
     _RegexpStrategyRule.__init__(self, pattern, ExcludedSymbol)
 
   def log(self, symbol):
-    Log().verbose(
+    logger.verbose(
         'Excluding symbol %s because it matches regexp "%s".'
         % (symbol, self.regexp.pattern,)
         )
@@ -156,7 +156,7 @@ class ExcludeTrivialImportBranchRule(StrategyRule):
       return symbol
     if stats.tag_create_count == 0 \
           and stats.branch_create_count == stats.trivial_import_count:
-      Log().verbose(
+      logger.verbose(
           'Excluding branch %s because it is a trivial import branch.'
           % (symbol,)
           )
@@ -177,7 +177,7 @@ class ExcludeVendorBranchRule(StrategyRule):
       return symbol
     if stats.tag_create_count == 0 \
           and stats.branch_create_count == stats.pure_ntdb_count:
-      Log().verbose(
+      logger.verbose(
           'Excluding branch %s because it is a pure vendor branch.'
           % (symbol,)
           )
@@ -198,14 +198,14 @@ class UnambiguousUsageRule(StrategyRule):
       # Can't decide
       return symbol
     elif is_branch:
-      Log().verbose(
+      logger.verbose(
           'Converting symbol %s as a branch because it is always used '
           'as a branch.'
           % (symbol,)
           )
       return Branch(symbol)
     elif is_tag:
-      Log().verbose(
+      logger.verbose(
           'Converting symbol %s as a tag because it is always used '
           'as a tag.'
           % (symbol,)
@@ -223,7 +223,7 @@ class BranchIfCommitsRule(StrategyRule):
     if isinstance(symbol, (Trunk, TypedSymbol)):
       return symbol
     elif stats.branch_commit_count > 0:
-      Log().verbose(
+      logger.verbose(
           'Converting symbol %s as a branch because there are commits on it.'
           % (symbol,)
           )
@@ -242,14 +242,14 @@ class HeuristicStrategyRule(StrategyRule):
     if isinstance(symbol, (Trunk, TypedSymbol)):
       return symbol
     elif stats.tag_create_count >= stats.branch_create_count:
-      Log().verbose(
+      logger.verbose(
           'Converting symbol %s as a tag because it is more often used '
           'as a tag.'
           % (symbol,)
           )
       return Tag(symbol)
     else:
-      Log().verbose(
+      logger.verbose(
           'Converting symbol %s as a branch because it is more often used '
           'as a branch.'
           % (symbol,)
@@ -257,25 +257,45 @@ class HeuristicStrategyRule(StrategyRule):
       return Branch(symbol)
 
 
-class AllBranchRule(StrategyRule):
+class _CatchAllRule(StrategyRule):
+  """Base class for catch-all rules.
+
+  Usually this rule will appear after a list of more careful rules
+  (including a general rule like UnambiguousUsageRule) and will
+  therefore only apply to the symbols not handled earlier."""
+
+  def __init__(self, action):
+    self._action = action
+
+  def log(self, symbol):
+    raise NotImplementedError()
+
+  def get_symbol(self, symbol, stats):
+    if isinstance(symbol, (Trunk, TypedSymbol)):
+      return symbol
+    else:
+      self.log(symbol)
+      return self._action(symbol)
+
+
+class AllBranchRule(_CatchAllRule):
   """Convert all symbols as branches.
 
   Usually this rule will appear after a list of more careful rules
   (including a general rule like UnambiguousUsageRule) and will
   therefore only apply to the symbols not handled earlier."""
 
-  def get_symbol(self, symbol, stats):
-    if isinstance(symbol, (Trunk, TypedSymbol)):
-      return symbol
-    else:
-      Log().verbose(
-          'Converting symbol %s as a branch because no other rules applied.'
-          % (symbol,)
-          )
-      return Branch(symbol)
+  def __init__(self):
+    _CatchAllRule.__init__(self, Branch)
+
+  def log(self, symbol):
+    logger.verbose(
+        'Converting symbol %s as a branch because no other rules applied.'
+        % (symbol,)
+        )
 
 
-class AllTagRule(StrategyRule):
+class AllTagRule(_CatchAllRule):
   """Convert all symbols as tags.
 
   We don't worry about conflicts here; they will be caught later by
@@ -285,15 +305,30 @@ class AllTagRule(StrategyRule):
   (including a general rule like UnambiguousUsageRule) and will
   therefore only apply to the symbols not handled earlier."""
 
-  def get_symbol(self, symbol, stats):
-    if isinstance(symbol, (Trunk, TypedSymbol)):
-      return symbol
-    else:
-      Log().verbose(
-          'Converting symbol %s as a tag because no other rules applied.'
-          % (symbol,)
-          )
-      return Tag(symbol)
+  def __init__(self):
+    _CatchAllRule.__init__(self, Tag)
+
+  def log(self, symbol):
+    logger.verbose(
+        'Converting symbol %s as a tag because no other rules applied.'
+        % (symbol,)
+        )
+
+
+class AllExcludedRule(_CatchAllRule):
+  """Exclude all symbols.
+
+  Usually this rule will appear after a list of more careful rules
+  (including a SymbolHintsFileRule or several ManualSymbolRules)
+  and will therefore only apply to the symbols not handled earlier."""
+
+  def __init__(self):
+    _CatchAllRule.__init__(self, ExcludedSymbol)
+
+  def log(self, symbol):
+    logger.verbose(
+        'Excluding symbol %s by catch-all rule.' % (symbol,)
+        )
 
 
 class TrunkPathRule(StrategyRule):
@@ -369,10 +404,10 @@ class HeuristicPreferredParentRule(StrategyRule):
     if isinstance(symbol, TypedSymbol) and symbol.preferred_parent_id is None:
       preferred_parent = self._get_preferred_parent(stats)
       if preferred_parent is None:
-        Log().verbose('%s has no preferred parent' % (symbol,))
+        logger.verbose('%s has no preferred parent' % (symbol,))
       else:
         symbol.preferred_parent_id = preferred_parent.id
-        Log().verbose(
+        logger.verbose(
             'The preferred parent of %s is %s' % (symbol, preferred_parent,)
             )
 
@@ -411,7 +446,7 @@ class ManualTrunkRule(StrategyRule):
 
 
 def convert_as_branch(symbol):
-  Log().verbose(
+  logger.verbose(
       'Converting symbol %s as a branch because of manual setting.'
       % (symbol,)
       )
@@ -419,7 +454,7 @@ def convert_as_branch(symbol):
 
 
 def convert_as_tag(symbol):
-  Log().verbose(
+  logger.verbose(
       'Converting symbol %s as a tag because of manual setting.'
       % (symbol,)
       )
@@ -427,7 +462,7 @@ def convert_as_tag(symbol):
 
 
 def exclude(symbol):
-  Log().verbose(
+  logger.verbose(
       'Excluding symbol %s because of manual setting.'
       % (symbol,)
       )
@@ -583,89 +618,92 @@ class SymbolHintsFileRule(StrategyRule):
     self._rules = []
 
     f = open(self.filename, 'r')
-    for l in f:
-      l = l.rstrip()
-      s = l.lstrip()
-      if self.comment_re.match(s):
-        continue
-      fields = s.split()
+    try:
+      for l in f:
+        l = l.rstrip()
+        s = l.lstrip()
+        if self.comment_re.match(s):
+          continue
+        fields = s.split()
 
-      if len(fields) < 3:
-        raise FatalError(
-            'The following line in "%s" cannot be parsed:\n    "%s"'
-            % (self.filename, l,)
-            )
+        if len(fields) < 3:
+          raise FatalError(
+              'The following line in "%s" cannot be parsed:\n    "%s"'
+              % (self.filename, l,)
+              )
 
-      project_id = fields.pop(0)
-      symbol_name = fields.pop(0)
-      conversion = fields.pop(0)
+        project_id = fields.pop(0)
+        symbol_name = fields.pop(0)
+        conversion = fields.pop(0)
 
-      if fields:
-        svn_path = fields.pop(0)
-        if svn_path == '.':
+        if fields:
+          svn_path = fields.pop(0)
+          if svn_path == '.':
+            svn_path = None
+          elif svn_path[0] == '/':
+            svn_path = svn_path[1:]
+        else:
           svn_path = None
-        elif svn_path[0] == '/':
-          svn_path = svn_path[1:]
-      else:
-        svn_path = None
 
-      if fields:
-        parent_lod_name = fields.pop(0)
-      else:
-        parent_lod_name = '.'
-
-      if fields:
-        raise FatalError(
-            'The following line in "%s" cannot be parsed:\n    "%s"'
-            % (self.filename, l,)
-            )
-
-      if project_id == '.':
-        project_id = None
-      else:
-        try:
-          project_id = int(project_id)
-        except ValueError:
-          raise FatalError(
-              'Illegal project_id in the following line:\n    "%s"' % (l,)
-              )
-
-      if symbol_name == '.trunk.':
-        if conversion not in ['.', 'trunk']:
-          raise FatalError('Trunk cannot be converted as a different type')
-
-        if parent_lod_name != '.':
-          raise FatalError('Trunk\'s parent cannot be set')
-
-        if svn_path is None:
-          # This rule doesn't do anything:
-          pass
+        if fields:
+          parent_lod_name = fields.pop(0)
         else:
-          self._rules.append(ManualTrunkRule(project_id, svn_path))
+          parent_lod_name = '.'
 
-      else:
-        try:
-          conversion = self.conversion_map[conversion]
-        except KeyError:
+        if fields:
           raise FatalError(
-              'Illegal conversion in the following line:\n    "%s"' % (l,)
+              'The following line in "%s" cannot be parsed:\n    "%s"'
+              % (self.filename, l,)
               )
 
-        if parent_lod_name == '.':
-          parent_lod_name = None
-
-        if conversion is None \
-               and svn_path is None \
-               and parent_lod_name is None:
-          # There is nothing to be done:
-          pass
+        if project_id == '.':
+          project_id = None
         else:
-          self._rules.append(
-              ManualSymbolRule(
-                  project_id, symbol_name,
-                  conversion, svn_path, parent_lod_name
-                  )
-              )
+          try:
+            project_id = int(project_id)
+          except ValueError:
+            raise FatalError(
+                'Illegal project_id in the following line:\n    "%s"' % (l,)
+                )
+
+        if symbol_name == '.trunk.':
+          if conversion not in ['.', 'trunk']:
+            raise FatalError('Trunk cannot be converted as a different type')
+
+          if parent_lod_name != '.':
+            raise FatalError('Trunk\'s parent cannot be set')
+
+          if svn_path is None:
+            # This rule doesn't do anything:
+            pass
+          else:
+            self._rules.append(ManualTrunkRule(project_id, svn_path))
+
+        else:
+          try:
+            conversion = self.conversion_map[conversion]
+          except KeyError:
+            raise FatalError(
+                'Illegal conversion in the following line:\n    "%s"' % (l,)
+                )
+
+          if parent_lod_name == '.':
+            parent_lod_name = None
+
+          if conversion is None \
+                 and svn_path is None \
+                 and parent_lod_name is None:
+            # There is nothing to be done:
+            pass
+          else:
+            self._rules.append(
+                ManualSymbolRule(
+                    project_id, symbol_name,
+                    conversion, svn_path, parent_lod_name
+                    )
+                )
+    finally:
+      f.close()
 
     for rule in self._rules:
       rule.start(symbol_statistics)
