@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2006-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2006-2009 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -20,10 +20,11 @@
 import os
 import re
 
-from cvs2svn_lib.log import Log
+from cvs2svn_lib.log import logger
 from cvs2svn_lib.common import FatalError
 from cvs2svn_lib.common import IllegalSVNPathError
 from cvs2svn_lib.common import normalize_svn_path
+from cvs2svn_lib.common import is_branch_revision_number
 
 
 class SymbolTransform:
@@ -138,9 +139,10 @@ class SymbolMapper(SymbolTransform):
   def __setitem__(self, (cvs_filename, symbol_name, revision), new_name):
     """Set a mapping for a particular file, symbol, and revision."""
 
+    cvs_filename = os.path.normcase(os.path.normpath(cvs_filename))
     key = (cvs_filename, symbol_name, revision)
     if key in self._map:
-      Log().warn(
+      logger.warn(
           'Overwriting symbol transform for\n'
           '    filename=%r symbol=%s revision=%s'
           % (cvs_filename, symbol_name, revision,)
@@ -148,8 +150,11 @@ class SymbolMapper(SymbolTransform):
     self._map[key] = new_name
 
   def transform(self, cvs_file, symbol_name, revision):
+    # cvs_file.rcs_path is guaranteed to already be normalised the way
+    # os.path.normpath() normalises paths.  No need to call it again.
+    cvs_filename = os.path.normcase(cvs_file.rcs_path)
     return self._map.get(
-        (cvs_file.filename, symbol_name, revision), symbol_name
+        (cvs_filename, symbol_name, revision), symbol_name
         )
 
 
@@ -185,8 +190,9 @@ class SubtreeSymbolMapper(SymbolTransform):
       symbol_map = {}
       self._map[symbol_name] = symbol_map
 
+    cvs_path = os.path.normcase(os.path.normpath(cvs_path))
     if cvs_path in symbol_map:
-      Log().warn(
+      logger.warn(
           'Overwriting symbol transform for\n'
           '    directory=%r symbol=%s'
           % (cvs_path, symbol_name,)
@@ -200,7 +206,9 @@ class SubtreeSymbolMapper(SymbolTransform):
       # No rules for that symbol name
       return symbol_name
 
-    cvs_path = cvs_file.filename
+    # cvs_file.rcs_path is guaranteed to already be normalised the way
+    # os.path.normpath() normalises paths.  No need to call it again.
+    cvs_path = os.path.normcase(cvs_file.rcs_path)
     while True:
       try:
         return symbol_map[cvs_path]
@@ -213,3 +221,125 @@ class SubtreeSymbolMapper(SymbolTransform):
           cvs_path = new_cvs_path
 
 
+class IgnoreSymbolTransform(SymbolTransform):
+  """Ignore symbols matching a specified regular expression."""
+
+  def __init__(self, pattern):
+    """Create an SymbolTransform that ignores symbols matching PATTERN.
+
+    PATTERN is a regular expression that should match the whole symbol
+    name."""
+
+    self.pattern = re.compile('^' + pattern + '$')
+
+  def transform(self, cvs_file, symbol_name, revision):
+    if self.pattern.match(symbol_name):
+      return None
+    else:
+      return symbol_name
+
+
+class SubtreeSymbolTransform(SymbolTransform):
+  """A wrapper around another SymbolTransform, that limits it to a
+  specified subtree."""
+
+  def __init__(self, cvs_path, inner_symbol_transform):
+    """Constructor.
+
+    CVS_PATH is the path in the repository.  INNER_SYMBOL_TRANSFORM is
+    the SymbolTransform to wrap."""
+
+    assert isinstance(cvs_path, str)
+    self.__subtree = os.path.normcase(os.path.normpath(cvs_path))
+    self.__subtree_len = len(self.__subtree)
+    self.__inner = inner_symbol_transform
+
+  def __does_rule_apply_to(self, cvs_file):
+    #
+    # NOTE: This turns out to be a hot path through the code.
+    #
+    # It used to use logic similar to SubtreeSymbolMapper.transform().  And
+    # it used to take 44% of cvs2svn's total runtime on one real-world test.
+    # Now it's been optimized, it takes about 2%.
+    #
+    # This is called about:
+    #   (num_files * num_symbols_per_file * num_subtree_symbol_transforms)
+    # times.  On a large repository with several of these transforms,
+    # that can exceed 100,000,000 calls.
+    #
+
+    # cvs_file.rcs_path is guaranteed to already be normalised the way
+    # os.path.normpath() normalises paths.  So we don't need to call
+    # os.path.normpath() again.  (The os.path.normpath() function does
+    # quite a lot, so it's expensive).
+    #
+    # os.path.normcase is a no-op on POSIX systems (and therefore fast).
+    # Even on Windows it's only a memory allocation and case-change, it
+    # should be quite fast.
+    cvs_path = os.path.normcase(cvs_file.rcs_path)
+
+    # Do most of the work in a single call, without allocating memory.
+    if not cvs_path.startswith(self.__subtree):
+      # Different prefix.
+      # This is the common "no match" case.
+      return False
+
+    if len(cvs_path) == self.__subtree_len:
+      # Exact match.
+      #
+      # This is quite rare, as self.__subtree is usually a directory and
+      # cvs_path is always a file.
+      return True
+
+    # We know cvs_path starts with self.__subtree, check the next character
+    # is a '/' (or if we're on Windows, a '\\').  If so, then cvs_path is a
+    # file under the self.__subtree directory tree, so we match.  If not,
+    # then it's not a match.
+    return cvs_path[self.__subtree_len] == os.path.sep
+
+  def transform(self, cvs_file, symbol_name, revision):
+    if self.__does_rule_apply_to(cvs_file):
+      return self.__inner.transform(cvs_file, symbol_name, revision)
+    else:
+      # Rule does not apply to that path; return symbol name unaltered.
+      return symbol_name
+
+
+class TagOnlyTransform(SymbolTransform):
+  """A wrapper around another SymbolTransform, that limits it to
+  CVS tags (not CVS branches)."""
+
+  def __init__(self, inner_symbol_transform):
+    """Constructor.
+
+    INNER_SYMBOL_TRANSFORM is the SymbolTransform to wrap."""
+    self.__inner = inner_symbol_transform
+
+  def transform(self, cvs_file, symbol_name, revision):
+    if is_branch_revision_number(revision):
+      # It's a branch
+      return symbol_name
+    else:
+      # It's a tag
+      return self.__inner.transform(cvs_file, symbol_name, revision)
+
+
+class BranchOnlyTransform(SymbolTransform):
+  """A wrapper around another SymbolTransform, that limits it to
+  CVS branches (not CVS tags)."""
+
+  def __init__(self, inner_symbol_transform):
+    """Constructor.
+
+    INNER_SYMBOL_TRANSFORM is the SymbolTransform to wrap."""
+    self.__inner = inner_symbol_transform
+
+  def transform(self, cvs_file, symbol_name, revision):
+    if is_branch_revision_number(revision):
+      # It's a branch
+      return self.__inner.transform(cvs_file, symbol_name, revision)
+    else:
+      # It's a tag
+      return symbol_name
+
+
