@@ -31,12 +31,15 @@ if sys.version_info[0] >= 3:
   from io import StringIO
 else:
   # Python <3.0
-  from StringIO import StringIO
+  from cStringIO import StringIO
 from xml.dom.minidom import parseString
 import base64
+import logging
 
 import svntest
 
+logger = logging.getLogger()
+
 # Tree Exceptions.
 
 # All tree exceptions should inherit from SVNTreeError
@@ -257,7 +260,7 @@ class SVNTreeNode:
       path = path[1:]
 
     line = prepend
-    line += "%-20s: Item(" % ("'%s'" % path)
+    line += "%-20s: Item(" % ("'%s'" % path.replace(os.sep, '/'))
     comma = False
 
     mime_type = self.props.get("svn:mime-type")
@@ -413,23 +416,6 @@ def add_elements_as_path(top_node, element_list):
     prev_node = new_node
 
 
-def compare_atts(a, b):
-  """Compare two dictionaries of attributes, A (actual) and B (expected).
-  If the attribute 'treeconflict' in B is missing or is 'None', ignore it.
-  Return 0 if the same, 1 otherwise."""
-  a = a.copy()
-  b = b.copy()
-  # Remove any attributes to ignore.
-  for att in ['treeconflict']:
-    if (att not in b) or (b[att] is None):
-      if att in a:
-        del a[att]
-      if att in b:
-        del b[att]
-  if a != b:
-    return 1
-  return 0
-
 # Helper for compare_trees
 def compare_file_nodes(a, b):
   """Compare two nodes, A (actual) and B (expected). Compare their names,
@@ -441,7 +427,34 @@ def compare_file_nodes(a, b):
     return 1
   if a.props != b.props:
     return 1
-  return compare_atts(a.atts, b.atts)
+  if a.atts == b.atts:
+    # No fixes necessary
+    return 0
+
+  # Fix a pre-WC-NG assumptions in our testsuite
+  if (b.atts == {'status': 'A ', 'wc_rev': '0'})     \
+     and (a.atts == {'status': 'A ', 'wc_rev': '-'}):
+    return 0
+  return 1
+
+# Helper for compare_trees
+def compare_dir_nodes(a, b):
+  """Compare two nodes, A (actual) and B (expected). Compare their names,
+  properties and attributes, ignoring children.  Return 0 if the
+  same, 1 otherwise."""
+  if a.name != b.name:
+    return 1
+  if (a.props != b.props):
+    return 1
+  if (a.atts == b.atts):
+    # No fixes necessary
+    return 0
+
+  # Fix a pre-WC-NG assumptions in our testsuite
+  if (b.atts == {'status': 'A ', 'wc_rev': '0'})     \
+     and (a.atts == {'status': 'A ', 'wc_rev': '-'}):
+    return 0
+  return 1
 
 
 # Internal utility used by most build_tree_from_foo() routines.
@@ -559,7 +572,7 @@ def get_child(node, name):
   """If SVNTreeNode NODE contains a child named NAME, return child;
   else, return None. If SVNTreeNode is not a directory, exit completely."""
   if node.children == None:
-    print("Error: Foolish call to get_child.")
+    logger.error("Foolish call to get_child.")
     sys.exit(1)
   for n in node.children:
     if name == n.name:
@@ -571,8 +584,8 @@ def get_child(node, name):
 def default_singleton_handler(node, description):
   """Print SVNTreeNode NODE's name, describing it with the string
   DESCRIPTION, then raise SVNTreeUnequal."""
-  print("Couldn't find node '%s' in %s tree" % (node.name, description))
-  node.pprint()
+  logger.warn("Couldn't find node '%s' in %s tree" % (node.name, description))
+  logger.warn(str(node))
   raise SVNTreeUnequal
 
 # A test helper function implementing the singleton_handler_a API.
@@ -589,8 +602,8 @@ def detect_conflict_files(node, extra_files):
       break
   else:
     msg = "Encountered unexpected disk path '" + node.name + "'"
-    print(msg)
-    node.pprint()
+    logger.warn(msg)
+    logger.warn(str(node))
     raise SVNTreeUnequal(msg)
 
 ###########################################################################
@@ -624,17 +637,20 @@ def compare_trees(label,
 
   def display_nodes(a, b):
     'Display two nodes, expected and actual.'
-    print("=============================================================")
-    print("Expected '%s' and actual '%s' in %s tree are different!"
-          % (b.name, a.name, label))
-    print("=============================================================")
-    print("EXPECTED NODE TO BE:")
-    print("=============================================================")
-    b.pprint()
-    print("=============================================================")
-    print("ACTUAL NODE FOUND:")
-    print("=============================================================")
-    a.pprint()
+    o = StringIO()
+    o.write("=============================================================\n")
+    o.write("Expected '%s' and actual '%s' in %s tree are different!\n"
+                % (b.name, a.name, label))
+    o.write("=============================================================\n")
+    o.write("EXPECTED NODE TO BE:\n")
+    o.write("=============================================================\n")
+    b.pprint(o)
+    o.write("=============================================================\n")
+    o.write("ACTUAL NODE FOUND:\n")
+    o.write("=============================================================\n")
+    a.pprint(o)
+    logger.warn(o.getvalue())
+    o.close()
 
   # Setup singleton handlers
   if singleton_handler_a is None:
@@ -657,8 +673,7 @@ def compare_trees(label,
       raise SVNTypeMismatch
     # They're both directories.
     else:
-      # First, compare the directories' two hashes.
-      if (a.props != b.props) or compare_atts(a.atts, b.atts):
+      if compare_dir_nodes(a, b):
         display_nodes(a, b)
         raise SVNTreeUnequal
 
@@ -681,21 +696,21 @@ def compare_trees(label,
         if b_child not in accounted_for:
           singleton_handler_b(b_child, b_baton)
   except SVNTypeMismatch:
-    print('Unequal Types: one Node is a file, the other is a directory')
+    logger.warn('Unequal Types: one Node is a file, the other is a directory')
     raise SVNTreeUnequal
   except IndexError:
-    print("Error: unequal number of children")
+    logger.warn("Error: unequal number of children")
     raise SVNTreeUnequal
   except SVNTreeUnequal:
     if a.name != root_node_name:
-      print("Unequal at node %s" % a.name)
+      logger.warn("Unequal at node %s" % a.name)
     raise
 
 
 
 # Visually show a tree's structure
 
-def dump_tree(n,indent=""):
+def _dump_tree(n,indent="",stream=sys.stdout):
   """Print out a nice representation of the structure of the tree in
   the SVNTreeNode N. Prefix each line with the string INDENT."""
 
@@ -703,18 +718,25 @@ def dump_tree(n,indent=""):
   tmp_children = sorted(n.children or [])
 
   if n.name == root_node_name:
-    print("%s%s" % (indent, "ROOT"))
+    stream.write("%s%s\n" % (indent, "ROOT"))
   else:
-    print("%s%s" % (indent, n.name))
+    stream.write("%s%s\n" % (indent, n.name))
 
   indent = indent.replace("-", " ")
   indent = indent.replace("+", " ")
   for i in range(len(tmp_children)):
     c = tmp_children[i]
     if i == len(tmp_children)-1:
-      dump_tree(c,indent + "  +-- ")
+      _dump_tree(c,indent + "  +-- ",stream)
     else:
-      dump_tree(c,indent + "  |-- ")
+      _dump_tree(c,indent + "  |-- ",stream)
+
+
+def dump_tree(n):
+    output = StringIO()
+    _dump_tree(n,stream=output)
+    logger.warn(output.getvalue())
+    output.close()
 
 
 def dump_tree_script__crawler(n, subtree="", stream=sys.stdout):
